//+------------------------------------------------------------------+
//|                                                        Gauss.mqh |
//|                              Copyright  2015, Vladimir Karputov |
//|                                           http://wmua.ru/slesar/ |
//+------------------------------------------------------------------+
#property copyright "Copyright  2015, Vladimir Karputov"
#property link      "http://wmua.ru/slesar/"
#property version   "1.0"
#property description "Blur image using a Gaussian filter"
#property description "Alglib library is required"
#include <Canvas\Canvas.mqh>
#include <Math\Alglib\alglib.mqh>
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class CGauss : public CCanvas
  {
private:
   CCanvas           m_shadow;               // canvas for drawing shadows
   uint              m_rad_blur;             // blur radius
   int               m_shadow_shift;         // shadow shift
   double            m_weights[];            // array of weights
   uint              m_color_blur_data[];    // array color blur
   uint              m_shadow_color;         // shadow color

public:
                     CGauss();
                    ~CGauss();
   //--- create/destroy
   bool              CreateBitmapLabel(const string name,const int x,const int y,
                                       const int width,const int height,ENUM_COLOR_FORMAT clrfmt=COLOR_FORMAT_XRGB_NOALPHA);
   //--- properties
   void              RadiusBlur(const uint radius) { m_rad_blur=radius;             }
   bool              ShadowShift(const int shift);
   void              SetShadowColor(const uint clr){ m_shadow_color=clr;            }
   //--- clear/fill color
   void              Erase(const uint clr=0);
   //--- draw primitives
   void              LineVertical(int x,int y1,int y2,const uint clr);
   void              LineHorizontal(int x1,int x2,int y,const uint clr);
   void              Line(int x1,int y1,int x2,int y2,const uint clr);
   void              Polyline(int &x[],int &y[],const uint clr);
   void              Polygon(int &x[],int &y[],const uint clr);
   void              Rectangle(int x1,int y1,int x2,int y2,const uint clr);
   void              Triangle(int x1,int y1,int x2,int y2,int x3,int y3,const uint clr);
   void              Circle(int x,int y,int r,const uint clr);
   //--- draw filled primitives
   void              FillRectangle(int x1,int y1,int x2,int y2,const uint clr);
   void              FillTriangle(int x1,int y1,int x2,int y2,int x3,int y3,const uint clr);
   void              FillPolygon(int &x[],int &y[],const uint clr);
   void              FillCircle(int x,int y,int r,const uint clr);
   void              FillEllipse(int x1,int y1,int x2,int y2,const uint clr);
   void              Fill(int x,int y,const uint clr);
   //--- for text
   bool              FontSet(const string name,const int size,const uint flags=0,const uint angle=0);
   bool              FontNameSet(string name);
   bool              FontSizeSet(int size);
   bool              FontFlagsSet(uint flags);
   bool              FontAngleSet(uint angle);
   void              FontGet(string &name,int &size,uint &flags,uint &angle);
   string            FontNameGet(void)                 const { return(m_fontname);  }
   int               FontSizeGet(void)                 const { return(m_fontsize);  }
   uint              FontFlagsGet(void)                const { return(m_fontflags); }
   uint              FontAngleGet(void)                const { return(m_fontangle); }
   void              TextOut(int x,int y,string text,const uint clr,uint alignment=0);
   int               TextWidth(const string text);
   int               TextHeight(const string text);
   void              TextSize(const string text,int &width,int &height);

private:
   bool              GetQuadratureWeights(const double mu0,const int n,double &w[]);
   //--- properties
   int               RadiusBlur(void) const              { return((int)m_rad_blur);       }
   int               ShadowShift(void) const             { return((int)m_shadow_shift);   }
   uint              GetShadowColor(void) const          { return(m_shadow_color);        }
   bool              GetArraysColorsBlur(const uint clr);
  };
//+------------------------------------------------------------------+
//| Constructor                                                      |
//+------------------------------------------------------------------+
CGauss::CGauss(void) : m_rad_blur(3),
                       m_shadow_shift(1),
                       m_shadow_color(0xC8A8A8A8)
  {
   if(!GetArraysColorsBlur(GetShadowColor()))
     {
      Print("Error Color Blur");
      return;
     }
   if(!ShadowShift(m_shadow_shift))
     {
      Print("Error Shadow Shift");
      return;
     }
  }
//+------------------------------------------------------------------+
//| Destructor                                                       |
//+------------------------------------------------------------------+
CGauss::~CGauss()
  {
   m_shadow.Destroy();
  }
//+------------------------------------------------------------------+
//| Create object on chart with attached dynamic resource            |
//+------------------------------------------------------------------+
bool CGauss::CreateBitmapLabel(const string name,const int x,const int y,
                               const int width,const int height,ENUM_COLOR_FORMAT clrfmt)
  {
   if(CCanvas::CreateBitmapLabel(name+"_shadow",x,y,width,height,clrfmt))
      if(m_shadow.CreateBitmapLabel(name+"_45",x+ShadowShift(),y+ShadowShift(),width,height,clrfmt))
        {
         m_shadow.Erase(ColorToARGB(clrNONE,0));
         m_shadow.Update();
         return(true);
        }
   return(false);
  }
//+------------------------------------------------------------------+
//| Shadow Shift                                                     |
//+------------------------------------------------------------------+
bool CGauss::ShadowShift(const int shift)
  {
   m_shadow_shift=shift;
   int  NNodes=RadiusBlur()*2+1;
   if(!GetQuadratureWeights(1,NNodes,m_weights))
      return(false);
   return(true);
  }
//+------------------------------------------------------------------+
//| Get Arrays Colors Blur                                           |
//+------------------------------------------------------------------+
bool CGauss::GetArraysColorsBlur(const uint clr)
  {
   int size=0;
   size=RadiusBlur()*3;
//---
   uchar a_temp=GETRGBA(clr);
   uchar r_temp=GETRGBR(clr);
   uchar g_temp=GETRGBG(clr);
   uchar b_temp=GETRGBB(clr);
//---
   uchar m_a_data[];             // array of Alpha channel
   uchar m_r_data[];             // array of Red channel
   uchar m_g_data[];             // array of Green channel
   uchar m_b_data[];             // array of Blue channel
//---
   if(ArrayResize(m_a_data,size)==-1)
     {
      //Print("Error setting the new size of an array of \"m_a_data\"");
      return(false);
     }
//---
   if(ArrayResize(m_r_data,size)==-1)
     {
      //Print("Error setting the new size of an array of \"m_r_data\"");
      return(false);
     }
//---
   if(ArrayResize(m_g_data,size)==-1)
     {
      //Print("Error setting the new size of an array of \"m_g_data\"");
      return(false);
     }
//---
   if(ArrayResize(m_b_data,size)==-1)
     {
      //Print("Error setting the new size of an array of \"m_b_data\"");
      return(false);
     }
//---
   int r=RadiusBlur();
   if(ArrayResize(m_color_blur_data,r)==-1)
     {
      //Print("Error setting the new size of an array of \"m_color_blur_data\"");
      return(false);
     }
   ArrayFill(m_a_data,r,size-r,0);
   ArrayFill(m_r_data,r,size-r,0);
   ArrayFill(m_g_data,r,size-r,0);
   ArrayFill(m_b_data,r,size-r,0);
//---
   for(int i=0;i<r;i++)
     {
      m_a_data[i]=a_temp;
      m_r_data[i]=r_temp;
      m_g_data[i]=g_temp;
      m_b_data[i]=b_temp;
     }
//---
   double dbl_a=0.0;
   double dbl_r=0.0;
   double dbl_g=0.0;
   double dbl_b=0.0;
//---
   uint     clr_temp;                     // color in ARGB format
   for(int i=0;i<r;i++)
     {
      for(int j=0;j<ArraySize(m_weights);j++)
        {
         dbl_a+=m_a_data[i+j]*m_weights[j];
         dbl_r+=m_r_data[i+j]*m_weights[j];
         dbl_g+=m_g_data[i+j]*m_weights[j];
         dbl_b+=m_b_data[i+j]*m_weights[j];
        }
      clr_temp=ARGB((uchar)MathRound(dbl_a),(uchar)MathRound(dbl_r),(uchar)MathRound(dbl_g),(uchar)MathRound(dbl_b));
      m_color_blur_data[i]=clr_temp;
      dbl_a=0.0;
      dbl_r=0.0;
      dbl_g=0.0;
      dbl_b=0.0;
     }
//---
   return(true);
  }
//+------------------------------------------------------------------+
//| Draw vertical line                                               |
//+------------------------------------------------------------------+
void CGauss::LineVertical(int x,int y1,int y2,const uint clr)
  {
   if(!GetArraysColorsBlur(GetShadowColor()))
     {
      Print("Error Color Blur");
      return;
     }
   for(int j=0;j<ArraySize(m_color_blur_data);j++)
     {
      m_shadow.LineVertical(x+j,y1+j,y2+j,m_color_blur_data[j]);
     }
   m_shadow.Update();
   CCanvas::LineVertical(x,y1,y2,clr);
  }
//+------------------------------------------------------------------+
//| Draw horizontal line                                             |
//+------------------------------------------------------------------+
void CGauss::LineHorizontal(int x1,int x2,int y,const uint clr)
  {
   if(!GetArraysColorsBlur(GetShadowColor()))
     {
      Print("Error Color Blur");
      return;
     }
   for(int j=0;j<ArraySize(m_color_blur_data);j++)
     {
      m_shadow.LineHorizontal(x1+j,x2+j,y+j,m_color_blur_data[j]);
     }
   m_shadow.Update();
   CCanvas::LineHorizontal(x1,x2,y,clr);
  }
//+------------------------------------------------------------------+
//| Draw line according to Bresenham's algorithm                     |
//+------------------------------------------------------------------+
void CGauss::Line(int x1,int y1,int x2,int y2,const uint clr)
  {
   if(!GetArraysColorsBlur(GetShadowColor()))
     {
      Print("Error Color Blur");
      return;
     }
   for(int j=0;j<ArraySize(m_color_blur_data);j++)
     {
      m_shadow.Line(x1+j,y1+j,x2+j,y2+j,m_color_blur_data[j]);
     }
   m_shadow.Update();
   CCanvas::Line(x1,y1,x2,y2,clr);
  }
//+------------------------------------------------------------------+
//| Draw polyline                                                    |
//+------------------------------------------------------------------+
void CGauss::Polyline(int &x[],int &y[],const uint clr)
  {
   if(!GetArraysColorsBlur(GetShadowColor()))
     {
      Print("Error Color Blur");
      return;
     }
   int x_shadow[],y_shadow[];
   if(ArrayResize(x_shadow,ArraySize(x))==-1)
     {
      //Print("Error setting the new size of an array of \"x_shadow\"");
      return;
     }
//---
   if(ArrayResize(y_shadow,ArraySize(y))==-1)
     {
      //Print("Error setting the new size of an array of \"y_shadow\"");
      return;
     }
//---
   for(int j=0;j<ArraySize(m_color_blur_data);j++)
     {
      for(int n=0;n<ArraySize(x_shadow);n++)
         x_shadow[n]=x[n]+j;
      for(int m=0;m<ArraySize(y_shadow);m++)
         y_shadow[m]=y[m]+j;
      m_shadow.Polyline(x_shadow,y_shadow,m_color_blur_data[j]);
     }
   m_shadow.Update();
   CCanvas::Polyline(x,y,clr);
  }
//+------------------------------------------------------------------+
//| Draw polygon                                                     |
//+------------------------------------------------------------------+
void CGauss::Polygon(int &x[],int &y[],const uint clr)
  {
   if(!GetArraysColorsBlur(GetShadowColor()))
     {
      Print("Error Color Blur");
      return;
     }
   int x_shadow[],y_shadow[];
   if(ArrayResize(x_shadow,ArraySize(x))==-1)
     {
      //Print("Error setting the new size of an array of \"x_shadow\"");
      return;
     }
//---
   if(ArrayResize(y_shadow,ArraySize(y))==-1)
     {
      //Print("Error setting the new size of an array of \"y_shadow\"");
      return;
     }
//---
   for(int j=0;j<ArraySize(m_color_blur_data);j++)
     {
      for(int n=0;n<ArraySize(x_shadow);n++)
         x_shadow[n]=x[n]+j;
      for(int m=0;m<ArraySize(y_shadow);m++)
         y_shadow[m]=y[m]+j;
      m_shadow.Polygon(x_shadow,y_shadow,m_color_blur_data[j]);
     }
   m_shadow.Update();
   CCanvas::Polygon(x,y,clr);
  }
//+------------------------------------------------------------------+
//| Draw rectangle                                                   |
//+------------------------------------------------------------------+
void CGauss::Rectangle(int x1,int y1,int x2,int y2,const uint clr)
  {
   if(!GetArraysColorsBlur(GetShadowColor()))
     {
      Print("Error Color Blur");
      return;
     }
   for(int j=0;j<ArraySize(m_color_blur_data);j++)
     {
      m_shadow.Rectangle(x1+j,y1+j,x2+j,y2+j,m_color_blur_data[j]);
     }
   m_shadow.Update();
   CCanvas::Rectangle(x1,y1,x2,y2,clr);
  }
//+------------------------------------------------------------------+
//| Draw triangle                                                    |
//+------------------------------------------------------------------+
void CGauss::Triangle(int x1,int y1,int x2,int y2,int x3,int y3,const uint clr)
  {
   if(!GetArraysColorsBlur(GetShadowColor()))
     {
      Print("Error Color Blur");
      return;
     }
   for(int j=0;j<ArraySize(m_color_blur_data);j++)
     {
      m_shadow.Triangle(x1+j,y1+j,x2+j,y2+j,x3+j,y3+j,m_color_blur_data[j]);
     }
   m_shadow.Update();
   CCanvas::Triangle(x1,y1,x2,y2,x3,y3,clr);
  }
//+------------------------------------------------------------------+
//| Draw circle                                                      |
//+------------------------------------------------------------------+
void CGauss::Circle(int x,int y,int r,const uint clr)
  {
   if(!GetArraysColorsBlur(GetShadowColor()))
     {
      Print("Error Color Blur");
      return;
     }
   for(int j=0;j<ArraySize(m_color_blur_data);j++)
     {
      m_shadow.Circle(x+j,y+j,r,m_color_blur_data[j]);
     }
   m_shadow.Update();
   CCanvas::Circle(x,y,r,clr);
  }
//+------------------------------------------------------------------+
//| Draw filled rectangle                                            |
//+------------------------------------------------------------------+
void CGauss::FillRectangle(int x1,int y1,int x2,int y2,const uint clr)
  {
   if(!GetArraysColorsBlur(GetShadowColor()))
     {
      Print("Error Color Blur");
      return;
     }
   for(int j=0;j<ArraySize(m_color_blur_data);j++)
     {
      m_shadow.FillRectangle(x1+j,y1+j,x2+j,y2+j,m_color_blur_data[j]);
     }
   m_shadow.Update();
   CCanvas::FillRectangle(x1,y1,x2,y2,clr);
  }
//+------------------------------------------------------------------+
//| Draw filled triangle                                             |
//+------------------------------------------------------------------+
void CGauss::FillTriangle(int x1,int y1,int x2,int y2,int x3,int y3,const uint clr)
  {
   if(!GetArraysColorsBlur(GetShadowColor()))
     {
      Print("Error Color Blur");
      return;
     }
   for(int j=0;j<ArraySize(m_color_blur_data);j++)
     {
      m_shadow.FillTriangle(x1+j,y1+j,x2+j,y2+j,x3+j,y3+j,m_color_blur_data[j]);
     }
   m_shadow.Update();
   CCanvas::FillTriangle(x1,y1,x2,y2,x3,y3,clr);
  }
//+------------------------------------------------------------------+
//| Draw filled poligon                                              |
//+------------------------------------------------------------------+
void CGauss::FillPolygon(int &x[],int &y[],const uint clr)
  {
   if(!GetArraysColorsBlur(GetShadowColor()))
     {
      Print("Error Color Blur");
      return;
     }
   int x_shadow[],y_shadow[];
   if(ArrayResize(x_shadow,ArraySize(x))==-1)
     {
      //Print("Error setting the new size of an array of \"x_shadow\"");
      return;
     }
//---
   if(ArrayResize(y_shadow,ArraySize(y))==-1)
     {
      //Print("Error setting the new size of an array of \"y_shadow\"");
      return;
     }
//---
   for(int j=0;j<ArraySize(m_color_blur_data);j++)
     {
      for(int n=0;n<ArraySize(x_shadow);n++)
         x_shadow[n]=x[n]+j;
      for(int m=0;m<ArraySize(y_shadow);m++)
         y_shadow[m]=y[m]+j;
      m_shadow.FillPolygon(x_shadow,y_shadow,m_color_blur_data[j]);
     }
   m_shadow.Update();
   CCanvas::FillPolygon(x,y,clr);
  }
//+------------------------------------------------------------------+
//| Draw filled circle                                               |
//+------------------------------------------------------------------+
void CGauss::FillCircle(int x,int y,int r,const uint clr)
  {
   if(!GetArraysColorsBlur(GetShadowColor()))
     {
      Print("Error Color Blur");
      return;
     }
   for(int j=0;j<ArraySize(m_color_blur_data);j++)
     {
      m_shadow.FillCircle(x+j,y+j,r,m_color_blur_data[j]);
     }
   m_shadow.Update();
   CCanvas::FillCircle(x,y,r,clr);
  }
//+------------------------------------------------------------------+
//| Draw filled ellipse                                              |
//+------------------------------------------------------------------+
void CGauss::FillEllipse(int x1,int y1,int x2,int y2,const uint clr)
  {
   if(!GetArraysColorsBlur(GetShadowColor()))
     {
      Print("Error Color Blur");
      return;
     }
   for(int j=0;j<ArraySize(m_color_blur_data);j++)
     {
      m_shadow.FillEllipse(x1+j,y1+j,x2+j,y2+j,m_color_blur_data[j]);
     }
   m_shadow.Update();
   CCanvas::FillEllipse(x1,y1,x2,y2,clr);
  }
//+------------------------------------------------------------------+
//| Fill closed region with color                                    |
//+------------------------------------------------------------------+
void CGauss::Fill(int x,int y,const uint clr)
  {
   if(!GetArraysColorsBlur(GetShadowColor()))
     {
      Print("Error Color Blur");
      return;
     }
   for(int j=0;j<ArraySize(m_color_blur_data);j++)
     {
      m_shadow.Fill(x+j,y+j,m_color_blur_data[j]);
     }
   m_shadow.Update();
   CCanvas::Fill(x,y,clr);
  }
//+------------------------------------------------------------------+
//| Set font                                                         |
//+------------------------------------------------------------------+
bool CGauss::FontSet(const string name,const int size,const uint flags,const uint angle)
  {
   if(!m_shadow.FontSet(name,size,flags,angle))
      return(false);
   if(!CCanvas::FontSet(name,size,flags,angle))
      return(false);
//--- succeed
   return(true);
  }
//+------------------------------------------------------------------+
//| Set fontname                                                     |
//+------------------------------------------------------------------+
bool CGauss::FontNameSet(string name)
  {
   if(!m_shadow.FontNameSet(name))
      return(false);
   if(!CCanvas::FontNameSet(name))
      return(false);
//--- succeed
   return(true);
  }
//+------------------------------------------------------------------+
//| Set fontsize                                                     |
//+------------------------------------------------------------------+
bool CGauss::FontSizeSet(int size)
  {
   if(!m_shadow.FontSizeSet(size))
      return(false);
   if(!CCanvas::FontSizeSet(size))
      return(false);
//--- succeed
   return(true);
  }
//+------------------------------------------------------------------+
//| Set fontflags                                                    |
//+------------------------------------------------------------------+
bool CGauss::FontFlagsSet(uint flags)
  {
   if(!m_shadow.FontFlagsSet(flags))
      return(false);
   if(!CCanvas::FontFlagsSet(flags))
      return(false);
//--- succeed
   return(true);
  }
//+------------------------------------------------------------------+
//| Set fontangle                                                    |
//+------------------------------------------------------------------+
bool CGauss::FontAngleSet(uint angle)
  {
   if(!m_shadow.FontAngleSet(angle))
      return(false);
   if(!CCanvas::FontAngleSet(angle))
      return(false);
//--- succeed
   return(true);
  }
//+------------------------------------------------------------------+
//| Get font params                                                  |
//+------------------------------------------------------------------+
void CGauss::FontGet(string &name,int &size,uint &flags,uint &angle)
  {
   m_shadow.FontGet(name,size,flags,angle);
   CCanvas::FontAngleSet(angle);
  }
//+------------------------------------------------------------------+
//| Out text                                                         |
//+------------------------------------------------------------------+
void CGauss::TextOut(int x,int y,string text,const uint clr,uint alignment)
  {
   if(!GetArraysColorsBlur(GetShadowColor()))
     {
      Print("Error Color Blur");
      return;
     }
   for(int j=0;j<ArraySize(m_color_blur_data);j++)
     {
      m_shadow.TextOut(x+j,y+j,text,m_color_blur_data[j],alignment);
     }
   m_shadow.Update();
   CCanvas::TextOut(x,y,text,clr,alignment);
  }
//+------------------------------------------------------------------+
//| Text width                                                       |
//+------------------------------------------------------------------+
int CGauss::TextWidth(const string text)
  {
   int w;
   m_shadow.TextWidth(text);
   w=CCanvas::TextWidth(text);
//--- result
   return(w);
  }
//+------------------------------------------------------------------+
//| Text height                                                      |
//+------------------------------------------------------------------+
int CGauss::TextHeight(const string text)
  {
   int h;
   m_shadow.TextHeight(text);
   h=CCanvas::TextHeight(text);
//--- result
   return(h);
  }
//+------------------------------------------------------------------+
//| Text rectangle                                                   |
//+------------------------------------------------------------------+
void CGauss::TextSize(const string text,int &width,int &height)
  {
   m_shadow.TextSize(text,width,height);
   CCanvas::TextSize(text,width,height);
  }
//+------------------------------------------------------------------+
//| Clear/Fill color                                                 |
//+------------------------------------------------------------------+
void CGauss::Erase(const uint clr)
  {
   m_shadow.Erase(ColorToARGB(clrNONE,0));
   CCanvas::Erase(clr);
  }
//+------------------------------------------------------------------+
//| Gets array of quadrature weights                                 |
//+------------------------------------------------------------------+
bool CGauss::GetQuadratureWeights(const double mu0,const int n,double &w[])
  {
   CAlglib alglib;            // static member of class CAlglib
   double      alp[];         // array alpha coefficients 
   double      bet[];         // array beta coefficients 
   ArrayResize(alp,n);
   ArrayResize(bet,n);
   ArrayInitialize(alp,1.0);  // initializes a numeric array alpha
   ArrayInitialize(bet,1.0);  // initializes a numeric array beta
//---
   double      out_x[];
   int         inf=0;
//| Info    -   error code:                                          |
//|                 * -3    internal eigenproblem solver hasn't      |
//|                         converged                                |
//|                 * -2    Beta[i]<=0                               |
//|                 * -1    incorrect N was passed                   |
//|                 *  1    OK                                       |
   alglib.GQGenerateRec(alp,bet,mu0,n,inf,out_x,w);
   if(inf!=1)
     {
      Print("Call error in CGaussQ::GQGenerateRec");
      return(false);
     }
   return(true);
  }
//+------------------------------------------------------------------+
